import React, { useEffect, useState } from "react";
import {
  Button,
  TextField,
  Dialog,
  DialogActions,
  DialogContent,
  DialogTitle,
  Typography,
  makeStyles,
  Tooltip
} from "@material-ui/core";
import { Autocomplete, Alert } from "@material-ui/lab";
import { useCSEntrySearch, useTagsAndCommitIds } from "../../api/SwaggerApi";
import { useTypingTimer } from "../common/SearchBoxFilter/TypingTimer";
import { formatDate, transformHostQuery } from "../common/Helper";

const useStyles = makeStyles((theme) => ({
  textField: {
    marginBottom: theme.spacing(1)
  },
  alert: {
    marginTop: theme.spacing(1)
  }
}));

export function IOCDeployDialog({
  open,
  setOpen,
  submitCallback,
  hasActiveDeployment,
  init = {},
  error,
  resetError
}) {
  function onError(message) {
    console.log(message);
  }

  const classes = useStyles();
  const [hosts, getHosts] = useCSEntrySearch();
  const [host, setHost] = useState(init);
  const [query, onHostKeyUp] = useTypingTimer({ interval: 500 });
  const [tagOrCommitId, getTagOrCommitId] = useTagsAndCommitIds(onError);
  const [gitRepo, setGitRepo] = useState(init.git || "");
  const [gitVersion, setGitVersion] = useState(init.version || "");
  const [queryGitVersion, onGitVersionKeyUp] = useTypingTimer({
    interval: 500
  });
  const gitProjectId = init.gitProjectId;

  const handleClose = () => {
    setOpen(false);
  };

  useEffect(
    () => getHosts({ query: transformHostQuery(`${query}`) }),
    [query, getHosts]
  );
  useEffect(() => {
    getTagOrCommitId(gitProjectId, queryGitVersion, false, "CONTAINS");
  }, [queryGitVersion, gitProjectId, getTagOrCommitId]);

  const onSubmit = (event) => {
    event.preventDefault();
    const { comment: commentText, git: gitText } = event.currentTarget.elements;
    const comment = commentText.value;
    const git = gitText.value;

    submitCallback({
      comment: comment,
      sourceUrl: git,
      sourceVersion: gitVersion,
      hostCSEntryId: host
        ? Number(host.csEntryHost.id)
        : init.csEntryHost
        ? init.csEntryHost.id
        : undefined
    });
  };

  // Creates, and formats tags/commitIds with dates into a table-format for using it in autocomplete
  function createOptionTags(option) {
    return (
      <table>
        <tr>
          <td style={{ width: "7em", maxWidth: "7em" }}>
            <Typography
              style={{
                fontFamily: "monospace",
                fontWeight: "bolder",
                overflow: "hidden",
                textOverflow: "ellipsis",
                whiteSpace: "nowrap"
              }}
            >
              {option.shortReference}
            </Typography>
          </td>
          <td style={{ width: "1em" }}>-</td>
          <td>
            <Typography
              style={{ fontFamily: "monospace", display: "inline-block" }}
            >
              {formatDate(option.commitDate)}
            </Typography>
          </td>
        </tr>
      </table>
    );
  }

  return (
    <Dialog
      open={open}
      onClose={handleClose}
    >
      <form onSubmit={onSubmit}>
        <DialogTitle id="form-dialog-title">
          {hasActiveDeployment ? "Deploy new revision" : "Deploy"}
        </DialogTitle>
        <DialogContent>
          <TextField
            autoComplete="off"
            className={classes.textField}
            id="comment"
            label="Deployment comment"
            variant="outlined"
            fullWidth
          />
          <TextField
            autoComplete="off"
            className={classes.textField}
            id="git"
            label="Git repository"
            variant="standard"
            defaultValue={init.git || ""}
            fullWidth
            onChange={(event) => {
              setGitRepo(event.target.value);
              resetError();
            }}
            disabled
            required
          />

          <Autocomplete
            className={classes.textField}
            autoHighlight
            id="version"
            options={tagOrCommitId}
            defaultValue={
              init.version
                ? {
                    reference: init.version,
                    shortReference: init.shortVersion
                      ? init.shortVersion
                      : init.version
                  }
                : null
            }
            getOptionLabel={(option) => {
              return option.shortReference;
            }}
            renderOption={(option) => {
              return (
                <Tooltip title={option.description}>
                  <div>{createOptionTags(option)}</div>
                </Tooltip>
              );
            }}
            onChange={(event, value, reason) => {
              setGitVersion(value?.reference);
              resetError();
              getTagOrCommitId(
                gitProjectId,
                value?.reference,
                false,
                "CONTAINS"
              );
            }}
            onInputChange={(event, value, reason) => {
              event && onGitVersionKeyUp(event.nativeEvent);
            }}
            disabled={!gitRepo || gitRepo.trim() === ""}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Git reference"
                variant="outlined"
                fullWidth
                required
              />
            )}
            autoSelect
          />

          <Autocomplete
            className={classes.textField}
            autoHighlight
            id="host"
            options={hosts.hostList}
            defaultValue={init}
            getOptionLabel={(option) => {
              return option?.csEntryHost?.fqdn;
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="host"
                variant="outlined"
                required
              />
            )}
            onChange={(event, value, reason) => {
              setHost(value);
              resetError();
            }}
            onInputChange={(event, value, reason) => {
              event && onHostKeyUp(event.nativeEvent);
            }}
            disabled={hasActiveDeployment}
            autoSelect
            filterOptions={(options, state) => options}
          />

          {hasActiveDeployment ? (
            <Typography variant="body">
              Hint: First undeploy IOC if you want to move to another host.
            </Typography>
          ) : (
            <></>
          )}

          {error ? (
            <Alert
              severity="error"
              className={classes.alert}
            >
              {error}
            </Alert>
          ) : (
            <></>
          )}
        </DialogContent>
        <DialogActions>
          <Button
            onClick={handleClose}
            color="primary"
          >
            Cancel
          </Button>
          <Button
            color="primary"
            variant="contained"
            type="submit"
            disabled={
              !host ||
              !gitVersion ||
              (hasActiveDeployment && init?.version === gitVersion)
            }
          >
            Deploy
          </Button>
        </DialogActions>
      </form>
    </Dialog>
  );
}
