import {
  Button,
  Link as MuiLink,
  Tooltip,
  Typography
} from "@material-ui/core";
import React, { useState, useEffect, useContext, useCallback } from "react";
import { IOCDetails } from "../../components/IOC/IOCDetails";
import { DeployIOC } from "../../components/IOC/DeployIOC";
import { UndeployIOC } from "../../components/IOC/UndeployIOC";
import { SimpleModal } from "../../components/common/SimpleModal/SimpleModal";
import {
  useUpdateAndDeployIoc,
  useCreateUndeployment,
  useJobById,
  userContext
} from "../../api/SwaggerApi";
import AlertMessages from "./AlertMessages";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import AccessControl from "../auth/AccessControl";
import { JobAsyncList } from "../Job/JobAsyncList";
import { DeploymentStatus } from "../../api/DataTypes";
import { IOCService } from "./IOCService";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";

export function IOCManage({
  ioc,
  getIOC,
  buttonDisabled,
  currentCommand,
  operations,
  operationsLoading,
  getOperations,
  setButtonDisabled,
  jobLazyParams,
  setJobLazyParams
}) {
  const { user } = useContext(userContext);
  const [state, setState] = useUrlState({
    log_open: "true"
  });

  const [deployDialogOpen, setDeployDialogOpen] = useState(false);
  const [undeployDialogOpen, setUndeployDialogOpen] = useState(false);
  const [deploymentJob, getDeploymentJobById] = useJobById();
  const [operationList, setOperationList] = useState(operations.operationsList);

  const rowsPerPage = [5, 20];

  useEffect(() => {
    if (ioc.activeDeployment?.awxJobId) {
      getDeploymentJobById(ioc.activeDeployment?.awxJobId);
    }
  }, [getDeploymentJobById, ioc.activeDeployment?.awxJobId]);

  useEffect(() => {
    setOperationList(operations.operationsList);
  }, [operations]);

  const closeDeployModal = () => {
    setDeployDialogOpen(false);
    getIOC();
    // history.push(window.location.pathname);
  };

  const closeUndeployModal = () => {
    setUndeployDialogOpen(false);
  };

  const getSubset = useCallback(
    (ioc) => {
      ioc = { ...ioc };
      const { sourceUrl: git } = ioc;

      // Show START/STOP components only when IOC was deployed SUCCESSFULLY
      const deploymentStatus = new DeploymentStatus(
        ioc.activeDeployment,
        deploymentJob
      );
      const showControls = deploymentStatus.wasSuccessful();

      let subset = {
        "IOC name": (
          <Typography>
            <MuiLink
              href={`${window.CCDB_ADDRESS}?name=${ioc.namingName}`}
              target="_blank"
              rel="noreferrer"
            >
              {ioc.namingName}
            </MuiLink>
          </Typography>
        ),
        git: (
          <Typography>
            <MuiLink
              href={git}
              target="_blank"
              rel="noreferrer"
            >
              {git}
            </MuiLink>
          </Typography>
        )
      };

      if (user) {
        subset["IOC Service Controls"] = showControls ? (
          <IOCService
            {...{
              ioc,
              currentCommand,
              getOperations,
              buttonDisabled,
              setButtonDisabled,
              jobLazyParams
            }}
          />
        ) : (
          "Can't be controlled, because IOC is not (yet) deployed succesfully"
        );
      }

      return subset;
    },
    [
      buttonDisabled,
      jobLazyParams,
      currentCommand,
      deploymentJob,
      getOperations,
      setButtonDisabled,
      user
    ]
  );

  let content = <></>;
  if (ioc) {
    console.log({ ...ioc });
    const managedIOC = { ...ioc };

    const formInit = {
      name: ioc.namingName,
      description: ioc.description,
      version: ioc.sourceVersion,
      shortVersion: ioc.sourceVersionShort,
      git: ioc.sourceUrl,
      gitProjectId: ioc.gitProjectId
    };

    if (ioc.activeDeployment) {
      formInit.csEntryHost = {
        fqdn: ioc.activeDeployment.host.fqdn,
        id: ioc.activeDeployment.host.csEntryId
      };
    }

    let disabledDebployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledDebployButtonTitle =
        "There is an ongoing operation, you can not deploy the IOC right now";
    }

    let disabledUndeployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledUndeployButtonTitle =
        "There is an ongoing operation, you can not undeploy the IOC right now";
    }

    content = (
      <>
        <IOCDetails
          ioc={managedIOC}
          getSubset={getSubset}
          alert={<AlertMessages alerts={managedIOC.alerts} />}
          buttons={
            <>
              <Tooltip title={disabledDebployButtonTitle}>
                <span>
                  <Button
                    variant="contained"
                    color="secondary"
                    onClick={() => {
                      setDeployDialogOpen(true);
                    }}
                    disabled={buttonDisabled || ioc.operationInProgress}
                  >
                    {ioc.activeDeployment ? "Deploy new revision" : "Deploy"}
                  </Button>
                </span>
              </Tooltip>
              {ioc.activeDeployment ? (
                <Tooltip title={disabledUndeployButtonTitle}>
                  <span>
                    <Button
                      variant="contained"
                      color="secondary"
                      style={{ marginRight: 20 }}
                      onClick={() => {
                        setUndeployDialogOpen(true);
                      }}
                      disabled={buttonDisabled || ioc.operationInProgress}
                    >
                      Undeploy
                    </Button>
                  </span>
                </Tooltip>
              ) : (
                <></>
              )}
            </>
          }
        />
        <AccessControl
          allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}
        >
          <SimpleAccordion
            summary="Log"
            expanded={deserialize(state.log_open)}
            onChange={(event, expanded) =>
              setState({ log_open: serialize(expanded) })
            }
          >
            <JobAsyncList
              jobs={operationList}
              setJobs={setOperationList}
              loading={operationsLoading}
              totalCount={operations.totalCount}
              lazyParams={jobLazyParams}
              setLazyParams={setJobLazyParams}
              rowsPerPage={rowsPerPage}
              deploymentId={ioc?.activeDeployment?.id}
            />
          </SimpleAccordion>
          <SimpleModal
            open={deployDialogOpen}
            setOpen={setDeployDialogOpen}
          >
            <DeployIOC
              open={deployDialogOpen}
              setOpen={setDeployDialogOpen}
              submitCallback={closeDeployModal}
              init={formInit}
              hook={useUpdateAndDeployIoc.bind(null, ioc.id)}
              hasActiveDeployment={ioc.activeDeployment}
            />
          </SimpleModal>
          <SimpleModal
            open={undeployDialogOpen}
            setOpen={setUndeployDialogOpen}
          >
            <UndeployIOC
              open={undeployDialogOpen}
              setOpen={setUndeployDialogOpen}
              submitCallback={closeUndeployModal}
              ioc={ioc}
              hook={useCreateUndeployment.bind(null, ioc.id)}
            />
          </SimpleModal>
        </AccessControl>
      </>
    );
  }

  return <>{content}</>;
}
